/* 
 * glass - the openGL Articulated Skeleton System
 *
 * Copyright (C) 2001  Robert Ancell <bob27@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <malloc.h>
#include <string.h>

#include "glass.h"

/* Returns an empty variable */
GlassVariable *_glassVariableNew(const char *name, GLfloat min, GLfloat max, GLfloat value)
{
  GlassVariable *variable = malloc(sizeof(GlassVariable));

  if(variable == NULL)
  {
    fprintf(stderr, "Glass Error: Unable to allocate space for a variable\n");
    return NULL;
  }

  variable->name = NULL;
  _glassVariableSetName(variable, name);

  variable->min = min;
  variable->max = max;
  variable->value = value;

  variable->next = NULL;

  return variable;
}

/* Set the variables name */
void _glassVariableSetName(GlassVariable *variable, const char *name)
{
  free(variable->name);
  if(name == NULL)
    variable->name = strdup("");
  else
    variable->name = strdup(name);
}

/* Set the variables name */
const char *_glassVariableGetName(GlassVariable *variable)
{
  return variable->name;
}

/* Set the value of a variable */
void _glassVariableSetValue(GlassVariable *variable, GLfloat value)
{
  variable->value = value;
  _glassVariableClip(variable);
}

/* Increment the value of a variable */
void _glassVariableIncValue(GlassVariable *variable, GLfloat dvalue)
{
  variable->value += dvalue;
  _glassVariableClip(variable);
}

/* Get the value of a variable */
GLfloat _glassVariableGetValue(GlassVariable *variable)
{
  return variable->value;
}

/* Get a pointer to the value of a variable */
GLfloat *_glassVariableGetValuev(GlassVariable *variable)
{
  return &variable->value;
}

/* Set the minimum of a variable */
void _glassVariableSetMin(GlassVariable *variable, GLfloat min)
{
  variable->min = min;
  _glassVariableClip(variable);
}

/* Get the minimum of a variable */
GLfloat _glassVariableGetMin(GlassVariable *variable)
{
  return variable->min;
}

/* Get a pointer to the minimum of a variable */
GLfloat *_glassVariableGetMinv(GlassVariable *variable)
{
  return &variable->min;
}

/* Set the max of a variable */
void _glassVariableSetMax(GlassVariable *variable, GLfloat max)
{
  variable->max = max;
  _glassVariableClip(variable);
}

/* Get the maximum of a variable */
GLfloat _glassVariableGetMax(GlassVariable *variable)
{
  return variable->max;
}

/* Get a pointer to the maximum of a variable */
GLfloat *_glassVariableGetMaxv(GlassVariable *variable)
{
  return &variable->max;
}

/* Clip the value of a variable so (min <= value <= max) */
void _glassVariableClip(GlassVariable *variable)
{
  if(variable->value < variable->min)
    variable->value = variable->min;
  else if(variable->value > variable->max)
    variable->value = variable->max;
}

/* Free a variable */
void _glassFreeVariable(GlassVariable *variable)
{
  free(variable->name);
  free(variable);
}
